@@ -6,6 +6,7 @@ import json
 import time
 import inspect
 import argparse
+import argcomplete
 import csv
 from typing import get_origin, get_args, Literal, Union
 
@@ -36,6 +37,7 @@ def cli_main():
     add("-z", dest="silent", action="store_true", help="Suppress all non-critical output")
     add("commands", nargs=argparse.REMAINDER, help="Project/Function command(s)")
 
+    argcomplete.autocomplete(parser)
     args = parser.parse_args()
 
     # Setup logging
@@ -57,7 +59,8 @@ def cli_main():
         name=args.username or "gw",
         project_path=args.projects,
         debug=args.debug,
-        wizard=args.wizard
+        wizard=args.wizard,
+        timed=args.timed
     )
 
     gw_local.verbose(
@@ -81,6 +84,8 @@ def cli_main():
         run_kwargs['client'] = args.client
     if args.server:
         run_kwargs['server'] = args.server
+    if args.timed:
+        run_kwargs['timed'] = True
     all_results, last_result = process(command_sources, **run_kwargs)
 
     # Resolve expression if requested
@@ -142,27 +147,40 @@ def process(command_sources, callback=None, **context):
     gw = Gateway(**context) if context else _global_gw
 
     def resolve_nested_object(root, tokens):
-        """Resolve a sequence of command tokens to a nested object (e.g. gw.project.module.func)."""
+        """Resolve a sequence of command tokens to a nested object (e.g. gw.project.module.func).
+
+        Returns a tuple ``(obj, remaining, path, error)`` where ``error`` is the
+        last ``AttributeError`` encountered while traversing the path. This lets
+        callers surface import failures instead of showing a generic 'No project'
+        message.
+        """
         path = []
         obj = root
+        last_error = None
 
         while tokens:
             normalized = normalize_token(tokens[0])
-            if hasattr(obj, normalized):
+            try:
                 obj = getattr(obj, normalized)
                 path.append(tokens.pop(0))
-            else:
-                # Try to resolve composite function names from remaining tokens
-                for i in range(len(tokens), 0, -1):
-                    joined = "_".join(normalize_token(t) for t in tokens[:i])
-                    if hasattr(obj, joined):
-                        obj = getattr(obj, joined)
-                        path.extend(tokens[:i])
-                        tokens[:] = tokens[i:]
-                        return obj, tokens, path
-                break  # No match found; exit lookup loop
+                continue
+            except AttributeError as e:
+                last_error = e
 
-        return obj, tokens, path
+            # Try to resolve composite function names from remaining tokens
+            for i in range(len(tokens), 0, -1):
+                joined = "_".join(normalize_token(t) for t in tokens[:i])
+                try:
+                    obj = getattr(obj, joined)
+                    path.extend(tokens[:i])
+                    tokens[:] = tokens[i:]
+                    return obj, tokens, path, last_error
+                except AttributeError as e:
+                    last_error = e
+                    continue
+            break  # No match found; exit lookup loop
+
+        return obj, tokens, path, last_error
 
     for chunk in command_sources:
         if not chunk:
@@ -187,10 +205,16 @@ def process(command_sources, callback=None, **context):
         if not chunk:
             continue
 
+        chunk = join_unquoted_kwargs(list(chunk))
+
         # Resolve nested project/function path
-        resolved_obj, func_args, path = resolve_nested_object(gw, list(chunk))
+        resolved_obj, func_args, path, attr_error = resolve_nested_object(
+            gw, list(chunk)
+        )
 
         if not callable(resolved_obj):
+            if attr_error is not None:
+                abort(str(attr_error))
             if hasattr(resolved_obj, '__functions__'):
                 show_functions(resolved_obj.__functions__)
             else:
@@ -245,23 +269,55 @@ def prepare(parsed_args, func_obj):
     var_kw_name = getattr(func_obj, "__var_keyword_name__", None)
     if var_kw_name:
         raw_items = getattr(parsed_args, var_kw_name, []) or []
-        it = iter(raw_items)
-        for token in it:
-            # support both “--key=value” and “--key value”
+        i = 0
+        while i < len(raw_items):
+            token = raw_items[i]
             if token.startswith("--") and "=" in token:
                 key, val = token[2:].split("=", 1)
+                i += 1
             elif token.startswith("--"):
                 key = token[2:]
-                try:
-                    val = next(it)
-                except StopIteration:
+                i += 1
+                if i >= len(raw_items):
                     abort(f"Expected a value after `{token}`")
+                val_tokens = []
+                while i < len(raw_items) and not raw_items[i].startswith("-"):
+                    val_tokens.append(raw_items[i])
+                    i += 1
+                if not val_tokens:
+                    abort(f"Expected a value after `{token}`")
+                val = " ".join(val_tokens)
             else:
-                abort(f"Invalid kwarg format `{token}`; expected `--key[=value]` or `--key value`.")
+                abort(
+                    f"Invalid kwarg format `{token}`; expected `--key[=value]` or `--key value`."
+                )
             extra_kwargs[key.replace("-", "_")] = val
 
     return func_args, {**func_kwargs, **extra_kwargs}
 
+def join_unquoted_kwargs(tokens: list[str]) -> list[str]:
+    """Combine values after ``--key`` up to the next dash token.
+
+    This allows passing multi-word strings without quoting as documented
+    in the "Unquoted Kwargs" section of :mod:`README`.
+    """
+    combined: list[str] = []
+    i = 0
+    while i < len(tokens):
+        token = tokens[i]
+        combined.append(token)
+        if token.startswith("--") and "=" not in token:
+            i += 1
+            value_parts = []
+            while i < len(tokens) and not tokens[i].startswith("-"):
+                value_parts.append(tokens[i])
+                i += 1
+            if value_parts:
+                combined.append(" ".join(value_parts))
+            continue
+        i += 1
+    return combined
+
 def chunk(args_commands):
     """Split args.commands into logical chunks without breaking quoted arguments."""
     chunks = []
@@ -416,9 +472,10 @@ def load_recipe(recipe_filename):
     non-whitespace characters are `--`, prepend the last full non-indented command prefix.
     
     Example:
-        web app setup --home reader
-            --project vbox --home upload
-            --project games.conway --home board --path conway
+        web app setup:
+            - web.site --home reader
+            - vbox --home upload
+            - games.conway --home board --path conway
         web server start-app --host 127.0.0.1 --port 8888
 
     This parses the indented lines as continuations of the previous non-indented command.
@@ -432,9 +489,24 @@ def load_recipe(recipe_filename):
 
     # --- Recipe file resolution (unchanged) ---
     if not os.path.isabs(recipe_filename):
-        candidate_names = [recipe_filename]
-        if not os.path.splitext(recipe_filename)[1]:
-            candidate_names += [f"{recipe_filename}.gwr", f"{recipe_filename}.txt"]
+        candidate_names = []
+        base_names = [recipe_filename]
+        if "." in recipe_filename:
+            base_names.append(recipe_filename.replace(".", "_"))
+            base_names.append(recipe_filename.replace(".", os.sep))
+
+        seen = set()
+        for base in base_names:
+            if base not in seen:
+                candidate_names.append(base)
+                seen.add(base)
+            if not os.path.splitext(base)[1]:
+                for ext in (".gwr", ".txt"):
+                    name = base + ext
+                    if name not in seen:
+                        candidate_names.append(name)
+                        seen.add(name)
+
         for name in candidate_names:
             recipe_path = gw.resource("recipes", name)
             if os.path.isfile(recipe_path):
@@ -450,14 +522,50 @@ def load_recipe(recipe_filename):
     
     deindented_lines = []
     last_prefix = ""
+    colon_prefix = None
+    colon_suffix = ""
     with open(recipe_path) as f:
         for raw_line in f:
             line = raw_line.rstrip("\n")
             stripped_line = line.lstrip()
             if not stripped_line:
+                colon_prefix = None
                 continue  # skip blank lines
             if stripped_line.startswith("#"):
                 comments.append(stripped_line)
+                colon_prefix = None
+                continue
+            if colon_prefix:
+                if stripped_line.startswith("- "):
+                    addition = stripped_line[1:].lstrip()
+                    line_to_add = colon_prefix + " " + addition
+                    if colon_suffix:
+                        line_to_add += " " + colon_suffix
+                    deindented_lines.append(line_to_add)
+                    continue
+                if stripped_line.startswith("--"):
+                    line_to_add = colon_prefix + " " + stripped_line
+                    if colon_suffix:
+                        line_to_add += " " + colon_suffix
+                    deindented_lines.append(line_to_add)
+                    continue
+                colon_prefix = None
+                colon_suffix = ""
+            if stripped_line.endswith(":"):
+                colon_prefix = stripped_line[:-1].rstrip()
+                colon_suffix = ""
+                last_prefix = colon_prefix
+                continue
+            # Detect colon inside line after a flag
+            no_comment = stripped_line.split("#", 1)[0].rstrip()
+            tokens = no_comment.split()
+            for idx, token in enumerate(tokens):
+                if token.endswith(":"):
+                    colon_prefix = " ".join(tokens[:idx + 1])[:-1].rstrip()
+                    colon_suffix = " ".join(tokens[idx + 1:])
+                    last_prefix = colon_prefix + (" " + colon_suffix if colon_suffix else "")
+                    break
+            if colon_prefix:
                 continue
             # Detect if line is indented and starts with '--'
             if line[:1].isspace() and stripped_line.startswith("--"):
