@@ -8,6 +8,7 @@ import logging
 import threading
 import importlib
 import functools
+import time
 from pathlib import Path
 
 from .envs import load_env, get_base_client, get_base_server
@@ -19,7 +20,7 @@ from .runner import Runner
 class Gateway(Resolver, Runner):
     _builtins = None  # Class-level: stores all discovered builtins only once
     _thread_local = threading.local()
-    defaults = {} 
+    defaults = {}
 
     Null = Null  # Null is a black-hole, assign with care.
 
@@ -28,10 +29,11 @@ class Gateway(Resolver, Runner):
     silent = False
     verbose = False
     wizard = False
+    timed = False
 
     def __init__(self, *,
                 client=None, server=None, name="gw", base_path=None, project_path=None,
-                verbose=None, silent=None, debug=None, wizard=None, quantity=1, **kwargs
+                verbose=None, silent=None, debug=None, wizard=None, timed=None, quantity=1, **kwargs
             ):
         self._cache = {}
         self._async_threads = []
@@ -45,7 +47,7 @@ class Gateway(Resolver, Runner):
 
         # --- Mode propagation logic: Set global flags via classmethod ---
         explicit_modes = {}
-        for flag in ('debug', 'silent', 'verbose', 'wizard'):
+        for flag in ('debug', 'silent', 'verbose', 'wizard', 'timed'):
             val = locals()[flag]
             if val is not None:
                 explicit_modes[flag] = val
@@ -55,6 +57,7 @@ class Gateway(Resolver, Runner):
         # Set instance mode flags to match class-level (which may have just changed)
         for flag in ('debug', 'silent', 'verbose', 'wizard'):
             setattr(self, f"{flag}_enabled", getattr(type(self), flag))
+        self.timed_enabled = timed if timed is not None else getattr(type(self), 'timed', False)
 
         # Instance-level log helpers: always use self.<flag>() to log
         self.debug = (lambda msg, *a, **k: self.logger.debug(msg, *a, stacklevel=2, **k)) if self.debug_enabled else Null
@@ -88,19 +91,19 @@ class Gateway(Resolver, Runner):
         # Load builtins ONCE, at class level
         if Gateway._builtins is None:
             builtins_module = importlib.import_module("gway.builtins")
-            Gateway._builtins = {
-                name: obj
-                for name, obj in inspect.getmembers(builtins_module)
-                if inspect.isfunction(obj)
-                and not name.startswith("_")
-                and inspect.getmodule(obj) == builtins_module
-            }
+            Gateway._builtins = {}
+            for name, obj in inspect.getmembers(builtins_module):
+                if not inspect.isfunction(obj) or name.startswith("_"):
+                    continue
+                mod = inspect.getmodule(obj)
+                if mod and mod.__name__.startswith("gway.builtins"):
+                    Gateway._builtins[name] = obj
 
     @classmethod
-    def update_modes(cls, *, debug=None, silent=None, verbose=None, wizard=None):
+    def update_modes(cls, *, debug=None, silent=None, verbose=None, wizard=None, timed=None):
         """Set global mode flags at the class level and on the global 'gw' instance, if present."""
         updated = {}
-        for flag, value in [('debug', debug), ('silent', silent), ('verbose', verbose), ('wizard', wizard)]:
+        for flag, value in [('debug', debug), ('silent', silent), ('verbose', verbose), ('wizard', wizard), ('timed', timed)]:
             if value is not None:
                 setattr(cls, flag, value)
                 updated[flag] = value
@@ -113,8 +116,8 @@ class Gateway(Resolver, Runner):
                 if isinstance(g, Gateway):
                     for flag, value in updated.items():
                         setattr(g, f"{flag}_enabled", value)
-                        # Update the instance method as well!
-                        setattr(g, flag, (lambda msg, *a, **k: g.logger.debug(msg, *a, stacklevel=2, **k)) if value else Null)
+                        if flag in ('debug', 'silent', 'verbose', 'wizard'):
+                            setattr(g, flag, (lambda msg, *a, **k: g.logger.debug(msg, *a, stacklevel=2, **k)) if value else Null)
         except Exception:
             pass
 
@@ -155,7 +158,7 @@ class Gateway(Resolver, Runner):
 
         result = set(discover_projects(projects_path))
         sorted_result = sorted(result)
-        self.verbose(f"Discovered projects: {sorted_result}", func="projects")
+        self.verbose(f"[projects] Discovered projects: {sorted_result}")
         return sorted_result
 
     def builtins(self):
@@ -169,12 +172,13 @@ class Gateway(Resolver, Runner):
         @functools.wraps(func_obj)
         def wrap(*args, **kwargs):
             try:
+                start_time = time.perf_counter() if self.timed_enabled else None
                 kwarg_txt = ', '.join(f"{k}='{v}'" for k, v in kwargs.items())
                 arg_txt = ', '.join(f"'{x}'" for x in args)
                 if kwarg_txt and arg_txt:
                     arg_txt = f"{arg_txt}, "
                 if not (is_builtin and not self.verbose):
-                    self.debug(f"-> {func_name}({arg_txt}{kwarg_txt})")
+                    self.verbose(f"-> {func_name}({arg_txt}{kwarg_txt})")
 
                 sig = inspect.signature(func_obj)
                 bound_args = sig.bind_partial(*args, **kwargs)
@@ -219,7 +223,7 @@ class Gateway(Resolver, Runner):
                         if (value is inspect.Parameter.empty or value is None) and name in type(self).defaults:
                             value = type(self).defaults[name]
                             if self.verbose:
-                                self.verbose(f"Injected default {name}={value!r} from Gateway.defaults", func="wrap_callable")
+                                self.verbose(f"[wrap_callable] Injected default {name}={value!r} from Gateway.defaults")
 
                     ann = param.annotation
                     if ann in (int, float, str, bool) and value is not None and not isinstance(value, ann):
@@ -254,6 +258,8 @@ class Gateway(Resolver, Runner):
                     )
                     self._async_threads.append(thread)
                     thread.start()
+                    if start_time is not None:
+                        self.log(f"[timed] {func_name} dispatch took {time.perf_counter() - start_time:.3f}s")
                     return f"ASYNC task started for {func_name}"
 
                 result = func_obj(*call_args, **call_kwargs)
@@ -266,6 +272,8 @@ class Gateway(Resolver, Runner):
                     )
                     self._async_threads.append(thread)
                     thread.start()
+                    if start_time is not None:
+                        self.log(f"[timed] {func_name} dispatch took {time.perf_counter() - start_time:.3f}s")
                     return f"ASYNC coroutine started for {func_name}"
 
                 # ---- Result storage logic ----
@@ -277,12 +285,15 @@ class Gateway(Resolver, Runner):
                     if any(word in subject for word in sensitive_keywords):
                         log_value = "[redacted]"
 
-                    if not self.verbose:
-                        self.debug(f"<- result['{subject}'] == {log_value}")
+                    self.verbose(f"<- result['{subject}'] == {log_value}")
                     self.results.insert(subject, result)
+
                     if isinstance(result, dict):
                         self.context.update(result)
 
+                if start_time is not None:
+                    self.log(f"[timed] {func_name} took {time.perf_counter() - start_time:.3f}s")
+
                 return result
 
             except Exception as e:
@@ -310,6 +321,10 @@ class Gateway(Resolver, Runner):
         try:
             project_obj = self.load_project(project_name=name)
             return project_obj
+        except FileNotFoundError as e:
+            # Avoid noisy stack traces for expected missing modules
+            self.debug(f"Project not found for attribute '{name}': {e}")
+            raise AttributeError(f"Unable to find GWAY attribute ({str(e)})")
         except Exception as e:
             self.exception(e)
             raise AttributeError(f"Unable to find GWAY attribute ({str(e)})")
@@ -323,7 +338,7 @@ class Gateway(Resolver, Runner):
             self.verbose(f"{project_name} <- Project('{base}')")
 
             def load_module_ns(py_path: str, dotted: str):
-                mod = self.load_py_file(py_path, dotted)
+                mod = self._load_py_file(py_path, dotted)
                 funcs = {}
                 for fname, obj in inspect.getmembers(mod, inspect.isfunction):
                     if not fname.startswith("_"):
@@ -365,6 +380,20 @@ class Gateway(Resolver, Runner):
             f"base_path/projects, env var, site-packages, and '{root}'."
         )
 
+    def find_project(self, *project_names: str, root: str = "projects"):
+        """Return the first successfully loaded project from ``project_names``.
+
+        Each name is passed to :meth:`load_project`. Projects that are not
+        found are ignored without logging any errors. ``None`` is returned if
+        none of the names can be loaded.
+        """
+        for proj in project_names:
+            try:
+                return self.load_project(proj, root=root)
+            except FileNotFoundError:
+                continue
+        return None
+
     def _projects_path(self):
         """
         Find the projects directory in source, install, or user-specified locations.
@@ -404,7 +433,11 @@ class Gateway(Resolver, Runner):
             "Tried base_path, GWAY_PROJECT_PATH, and package resources."
         )
     
-    def load_py_file(self, path: str, dotted_name: str):
+    def _load_py_file(self, path: str, dotted_name: str):
+        """
+        Don't manually use _load_py_file, instead simply access the proyect through Gateway:
+        gw.project.subproject.function(), where gw is any Gateway instance such as from gway import gw.
+        """
         module_name = dotted_name.replace(".", "_")
         spec = importlib.util.spec_from_file_location(module_name, path)
         if spec is None or spec.loader is None:
@@ -438,7 +471,7 @@ class Gateway(Resolver, Runner):
                 if subname == dir_basename:
                     continue  # defer loading the root file until later
                 dotted = f"{dotted_prefix}.{subname}"
-                mod = self.load_py_file(full, dotted)
+                mod = self._load_py_file(full, dotted)
                 sub_funcs = {}
                 for fname, obj in inspect.getmembers(mod, inspect.isfunction):
                     if not fname.startswith("_"):
@@ -451,7 +484,7 @@ class Gateway(Resolver, Runner):
         # 2. Load the root file (e.g., web/web.py) if present
         root_funcs = {}
         if os.path.isfile(root_file):
-            mod = self.load_py_file(root_file, dotted_prefix)
+            mod = self._load_py_file(root_file, dotted_prefix)
             for fname, obj in inspect.getmembers(mod, inspect.isfunction):
                 if not fname.startswith("_"):
                     root_funcs[fname] = self.wrap_callable(f"{dotted_prefix}.{fname}", obj)
