@@ -18,7 +18,7 @@ This virtual box (vbox) system allows users with admin access to upload and down
 files to/from a secure folder in the remote server. 
 
 In CLI/Recipes you may use:
-> [gway] web app setup --project vbox --home upload
+> [gway] web app setup vbox --home upload
 
 To deploy it and sey the home to [BASE_URL]/vbox/upload.
 """
@@ -30,6 +30,13 @@ _gc_thread_on = False
 VBOX_PATH = "work", "vbox"
 
 
+def _sanitize_filename(name: str) -> str:
+    """Return a safe filename by stripping path separators and unsafe chars."""
+    name = str(name)
+    name = name.replace("/", "").replace("\\", "").replace("..", "")
+    return "".join(c for c in name if c.isalnum() or c in "._-")
+
+
 def purge(*, all=False):
     """Manually purge expired vbox entries and remove their folders.
 
@@ -124,14 +131,15 @@ def view_uploads(*, vbid: str = None, timeout: int = 60, files: int = 4, email:
         uploaded_files = request.files.getlist("file")
         results = []
         for f in uploaded_files:
-            save_path = os.path.join(upload_dir, f.filename)
+            safe_name = _sanitize_filename(f.filename)
+            save_path = os.path.join(upload_dir, safe_name)
             try:
                 f.save(save_path)
-                results.append(f"Uploaded {f.filename}")
-                gw.info(f"Uploaded {f.filename} to {short}")
+                results.append(f"Uploaded {safe_name}")
+                gw.info(f"Uploaded {safe_name} to {short}")
             except Exception as e:
-                results.append(f"Error uploading {f.filename}: {e}")
-                gw.error(f"Issue uploading {f.filename} to {short}")
+                results.append(f"Error uploading {safe_name}: {e}")
+                gw.error(f"Issue uploading {safe_name} to {short}")
                 gw.exception(e)
 
         download_short_url = gw.web.app.build_url("download", vbid=short)
@@ -250,7 +258,7 @@ def open_remote(server_url: str = '[SERVER_URL]', *, path: str = 'vbox', email:
     Create a vbox on a remote system, retrieve the upload link from email, and store it locally.
     - server_url: Base URL of the remote server (e.g., 'https://example.com')
     - path:       Path on remote server where vbox upload is handled (default 'vbox')
-    - email:      Email address to receive the upload link (should be accessible by local mail.search)
+    - email:      Email address to receive the upload link (should be accessible by local mail.read)
     
     Returns: dict of stored record fields, or None if unsuccessful.
     """
@@ -287,7 +295,7 @@ def open_remote(server_url: str = '[SERVER_URL]', *, path: str = 'vbox', email:
 
     for attempt in range(max_wait // poll_interval):
         try:
-            result = gw.mail.search(subject_fragment)
+            result = gw.mail.read(subject_fragment)
             if result:
                 body, _ = result
                 match = re.search(access_url_pattern, body)
@@ -301,7 +309,7 @@ def open_remote(server_url: str = '[SERVER_URL]', *, path: str = 'vbox', email:
                         gw.info(f"[open_remote] Parsed vbid: {found_vbid}")
                         break
         except Exception as e:
-            gw.error(f"[open_remote] Error during mail.search: {e}")
+            gw.error(f"[open_remote] Error during mail.read: {e}")
         time.sleep(poll_interval)
 
     if not (found_url and found_vbid):
@@ -395,7 +403,8 @@ def poll_remote(server_url: str = '[SERVER_URL]', *, target='work/vbox/remote',
 
     def download_file(md5, name, mtime):
         # Download if missing or outdated
-        local_path = os.path.join(target, name)
+        safe_name = _sanitize_filename(name)
+        local_path = os.path.join(target, safe_name)
         # Only download if missing or mtime newer
         if os.path.exists(local_path):
             prev = os.path.getmtime(local_path)
@@ -406,16 +415,16 @@ def poll_remote(server_url: str = '[SERVER_URL]', *, target='work/vbox/remote',
         try:
             resp = requests.get(file_url, timeout=30)
             if resp.status_code == 304:
-                gw.info(f"[poll_remote] Skipped {name}: not modified")
+                gw.info(f"[poll_remote] Skipped {safe_name}: not modified")
                 return False
             resp.raise_for_status()
             with open(local_path, "wb") as f:
                 f.write(resp.content)
             os.utime(local_path, (mtime, mtime))  # Set mtime to match remote
-            gw.info(f"[poll_remote] Downloaded {name} ({md5})")
+            gw.info(f"[poll_remote] Downloaded {safe_name} ({md5})")
             return True
         except Exception as e:
-            gw.error(f"[poll_remote] Error downloading {name}: {e}")
+            gw.error(f"[poll_remote] Error downloading {safe_name}: {e}")
             return False
 
     # Main polling loop
@@ -455,7 +464,6 @@ def view_downloads(*hashes: tuple[str], vbid: str = None, modified_since=None, *
     - If modified_since is passed (as iso or epoch seconds), only send file if newer, else 304.
     """
 
-    # TODO: Support multiple hashes by checking them one by one. If the first doesn't exist, 
     # try the next and so forth. Give up when every hash fails to match. First matches is chosen first.
 
     gw.warning(f"Download view: {hashes=} {vbid=} {kwargs=}")
