@@ -1,6 +1,7 @@
 # file: tests/test_etron_ws.py
 
 import unittest
+from gway.builtins import is_test_flag
 import sys
 import subprocess
 import time
@@ -19,6 +20,7 @@ UNKNOWN_TAG = "ZZZZZZZZ"
 
 import signal
 
+@unittest.skipUnless(is_test_flag("ocpp"), "OCPP tests disabled")
 class EtronWebSocketTests(unittest.TestCase):
     @classmethod
     def setUpClass(cls):
@@ -107,7 +109,7 @@ class EtronWebSocketTests(unittest.TestCase):
         start = time.time()
         while time.time() - start < timeout:
             try:
-                with socket.create_connection(("localhost", port), timeout=1):
+                with socket.create_connection(("127.0.0.1", port), timeout=1):
                     return
             except OSError:
                 time.sleep(0.2)
@@ -122,7 +124,7 @@ class EtronWebSocketTests(unittest.TestCase):
 
     def test_websocket_connection(self):
         """Confirm we can connect to the OCPP server and receive BootNotification response."""
-        uri = "ws://localhost:19000/charger123?token=foo"
+        uri = "ws://127.0.0.1:19000/charger123?token=foo"
         async def run_ws_check():
             async with websockets.connect(uri, subprotocols=["ocpp1.6"], open_timeout=15) as websocket:
                 message_id = "boot-test"
@@ -141,7 +143,7 @@ class EtronWebSocketTests(unittest.TestCase):
     def test_authorize_valid_rfid(self):
         """RFID in allowlist with balance >=1 should be Accepted"""
         self._set_balance(KNOWN_GOOD_TAG, 100)
-        uri = "ws://localhost:19000/tester1?token=foo"
+        uri = "ws://127.0.0.1:19000/tester1?token=foo"
         async def run_authorize_check():
             async with websockets.connect(uri, subprotocols=["ocpp1.6"]) as websocket:
                 message_id = "auth-valid"
@@ -158,7 +160,7 @@ class EtronWebSocketTests(unittest.TestCase):
     def test_authorize_with_extra_fields(self):
         """RFID with additional fields in CDV still authorizes correctly"""
         self.__class__.gw_cdv_update(KNOWN_GOOD_TAG, balance="55", foo="bar", baz="qux")
-        uri = "ws://localhost:19000/tester2?token=foo"
+        uri = "ws://127.0.0.1:19000/tester2?token=foo"
         async def run_authorize_check():
             async with websockets.connect(uri, subprotocols=["ocpp1.6"]) as websocket:
                 message_id = "auth-extra"
@@ -175,7 +177,7 @@ class EtronWebSocketTests(unittest.TestCase):
     def test_authorize_low_balance(self):
         """RFID present but balance <1 should be Rejected"""
         self._set_balance(KNOWN_GOOD_TAG, 0)
-        uri = "ws://localhost:19000/tester3?token=foo"
+        uri = "ws://127.0.0.1:19000/tester3?token=foo"
         async def run_authorize_check():
             async with websockets.connect(uri, subprotocols=["ocpp1.6"]) as websocket:
                 message_id = "auth-lowbal"
@@ -192,7 +194,7 @@ class EtronWebSocketTests(unittest.TestCase):
     def test_authorize_admin_tag(self):
         """Admin tag should be accepted (if balance >=1)"""
         self._set_balance(ADMIN_TAG, 150)
-        uri = "ws://localhost:19000/admin?token=foo"
+        uri = "ws://127.0.0.1:19000/admin?token=foo"
         async def run_authorize_check():
             async with websockets.connect(uri, subprotocols=["ocpp1.6"]) as websocket:
                 message_id = "auth-admin"
@@ -208,7 +210,7 @@ class EtronWebSocketTests(unittest.TestCase):
 
     def test_authorize_unknown_rfid(self):
         """Unknown tag must be rejected"""
-        uri = "ws://localhost:19000/unknown?token=foo"
+        uri = "ws://127.0.0.1:19000/unknown?token=foo"
         async def run_authorize_check():
             async with websockets.connect(uri, subprotocols=["ocpp1.6"]) as websocket:
                 message_id = "auth-unknown"
@@ -227,8 +229,8 @@ class EtronWebSocketTests(unittest.TestCase):
         self._set_balance(KNOWN_GOOD_TAG, 100)
         self._set_balance(ADMIN_TAG, 0)
         uris = [
-            "ws://localhost:19000/chargerA?token=foo",
-            "ws://localhost:19000/chargerB?token=foo"
+            "ws://127.0.0.1:19000/chargerA?token=foo",
+            "ws://127.0.0.1:19000/chargerB?token=foo"
         ]
         async def run_concurrent():
             async def connect_and_auth(uri, idtag):
@@ -250,7 +252,7 @@ class EtronWebSocketTests(unittest.TestCase):
     def test_authorize_missing_balance(self):
         """If balance is missing, should be treated as 0 and Rejected."""
         self.__class__.gw_cdv_update(KNOWN_GOOD_TAG, user="test")  # No balance field!
-        uri = "ws://localhost:19000/missingbal?token=foo"
+        uri = "ws://127.0.0.1:19000/missingbal?token=foo"
         async def run_authorize_check():
             async with websockets.connect(uri, subprotocols=["ocpp1.6"]) as websocket:
                 message_id = "auth-missingbal"
@@ -266,7 +268,7 @@ class EtronWebSocketTests(unittest.TestCase):
 
     def test_server_ignores_callresult_messages(self):
         """Server should ignore valid [3, ...] CALLRESULT messages from client."""
-        uri = "ws://localhost:19000/callresult1?token=foo"
+        uri = "ws://127.0.0.1:19000/callresult1?token=foo"
         async def run_ignore_callresult():
             async with websockets.connect(uri, subprotocols=["ocpp1.6"]) as websocket:
                 message_id = "irrelevant-id"
@@ -286,7 +288,7 @@ class EtronWebSocketTests(unittest.TestCase):
 
     def test_server_ignores_callerror_messages(self):
         """Server should ignore valid [4, ...] CALLERROR messages from client."""
-        uri = "ws://localhost:19000/callerror1?token=foo"
+        uri = "ws://127.0.0.1:19000/callerror1?token=foo"
         async def run_ignore_callerror():
             async with websockets.connect(uri, subprotocols=["ocpp1.6"]) as websocket:
                 message_id = "irrelevant-id"
@@ -319,9 +321,9 @@ class EtronWebSocketTests(unittest.TestCase):
                 ]},
             ]
         }
-        pc = gw.ocpp.csms.power_consumed(tx1)
+        pc = gw.ocpp.power_consumed(tx1)
         self.assertAlmostEqual(pc, 2.0, places=2)
-        lm = gw.ocpp.csms.extract_meter(tx1)
+        lm = gw.ocpp.extract_meter(tx1)
         self.assertAlmostEqual(lm, 152.0, places=2)
 
         # Simulated transaction with MeterValues in Wh (should convert to kWh)
@@ -337,22 +339,22 @@ class EtronWebSocketTests(unittest.TestCase):
                 ]},
             ]
         }
-        pc = gw.ocpp.csms.power_consumed(tx2)
+        pc = gw.ocpp.power_consumed(tx2)
         self.assertAlmostEqual(pc, 2.5, places=2)
-        lm = gw.ocpp.csms.extract_meter(tx2)
+        lm = gw.ocpp.extract_meter(tx2)
         self.assertAlmostEqual(lm, 102.5, places=2)
 
         # Only meterStart/meterStop (no MeterValues)
         tx3 = {"meterStart": 123000, "meterStop": 124500, "MeterValues": []}
-        pc = gw.ocpp.csms.power_consumed(tx3)
+        pc = gw.ocpp.power_consumed(tx3)
         self.assertAlmostEqual(pc, 1.5, places=2)
-        lm = gw.ocpp.csms.extract_meter(tx3)
+        lm = gw.ocpp.extract_meter(tx3)
         self.assertAlmostEqual(lm, 124.5, places=2)
 
         # Edge: no data
-        pc = gw.ocpp.csms.power_consumed({})
+        pc = gw.ocpp.power_consumed({})
         self.assertEqual(pc, 0.0)
-        lm = gw.ocpp.csms.extract_meter({})
+        lm = gw.ocpp.extract_meter({})
         self.assertEqual(lm, "-")
 
         # Edge: MeterValues present but missing correct measurand
@@ -361,14 +363,39 @@ class EtronWebSocketTests(unittest.TestCase):
                 {"value": 12.34, "measurand": "NotEnergy", "unit": "kWh"}
             ]}
         ]}
-        pc = gw.ocpp.csms.power_consumed(tx4)
+        pc = gw.ocpp.power_consumed(tx4)
         self.assertEqual(pc, 0.0)
-        lm = gw.ocpp.csms.extract_meter(tx4)
+        lm = gw.ocpp.extract_meter(tx4)
         self.assertEqual(lm, "-")
 
+        # meterStart with MeterValues only (no meterStop)
+        tx5 = {
+            "meterStart": 500000,
+            "MeterValues": [
+                {"timestamp": 1, "sampledValue": [
+                    {"value": 500.0, "measurand": "Energy.Active.Import.Register", "unit": "kWh"}
+                ]},
+                {"timestamp": 2, "sampledValue": [
+                    {"value": 501.0, "measurand": "Energy.Active.Import.Register", "unit": "kWh"}
+                ]},
+            ]
+        }
+        pc = gw.ocpp.power_consumed(tx5)
+        self.assertAlmostEqual(pc, 1.0, places=2)
+        lm = gw.ocpp.extract_meter(tx5)
+        self.assertAlmostEqual(lm, 501.0, places=2)
+
+    def test_power_consumed_accepts_json(self):
+        """Helpers should parse JSON strings transparently."""
+        tx = {"meterStart": 0, "meterStop": 1000, "MeterValues": []}
+        js = json.dumps(tx)
+        self.assertAlmostEqual(gw.ocpp.power_consumed(js), 1.0, places=2)
+        self.assertAlmostEqual(gw.ocpp.extract_meter(js), 1.0, places=2)
+
+    @unittest.skipUnless(is_test_flag("ocpp"), "OCPP tests disabled")
     def test_remote_stop_transaction(self):
         """Dashboard Stop action triggers RemoteStopTransaction on the CP."""
-        uri = "ws://localhost:19000/stopper?token=foo"
+        uri = "ws://127.0.0.1:19000/stopper?token=foo"
 
         async def run_stop_check():
             async with websockets.connect(uri, subprotocols=["ocpp1.6"]) as ws:
@@ -400,6 +427,29 @@ class EtronWebSocketTests(unittest.TestCase):
 
         asyncio.run(run_stop_check())
 
+    def test_evcs_simulator_session_runs_without_recv_error(self):
+        """EVCS simulator should complete a short session without recv overlap errors."""
+        async def run_sim():
+            import io, contextlib
+            buf = io.StringIO()
+            with contextlib.redirect_stdout(buf):
+                await gw.ocpp.evcs.simulate_cp.__wrapped__(
+                    0,
+                    "127.0.0.1",
+                    19000,
+                    KNOWN_GOOD_TAG,
+                    "SIMTEST",
+                    1,
+                    1,
+                    1,
+                    1,
+                )
+            return buf.getvalue()
+
+        output = asyncio.run(run_sim())
+        self.assertNotIn("cannot call recv", output)
+        self.assertIn("Simulation ended", output)
+
 
 if __name__ == "__main__":
     unittest.main()
